#
# WiKicker
#
#  LinkMaker.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: LinkMaker.pm,v 1.7 2003/11/08 09:48:30 naney Exp $
#
#

package WiKicker::HTML::LinkMaker;

use 5.005;
use strict;
use WiKicker::URI;
use WiKicker::HTML;
use WiKicker::PageName;
use WiKicker::Time;

sub new {
  my ($class, %param) = @_;

  my $self = bless {}, $class;

  if (exists $param{env}) {
    my $env = $param{env};
    $self->script_url($env->wikicker_url);
    $self->db($env->db);
    $self->configuration($env->configuration);
    if ($env->user) {
      $self->timezone($env->user->timezone || 'GMT');
    }
    else {
      $self->timezone('GMT');
    }
  }

  return $self;
}

sub script_url {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{script_url} = $value;
  }
  return $self->{script_url};
}

sub timezone {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{timezone} = $value;
  }
  return $self->{timezone};
}

sub db {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{db} = $value;
  }
  return $self->{db};
}

sub configuration {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{configuration} = $value;
  }
  return $self->{configuration};
}

sub make_page_read_href {
  my ($self, $page_name) = @_;

  if (exists $self->{page_read_href}->{$page_name}) {
    return $self->{page_read_href}->{$page_name};
  }
  else {
    return $self->{page_read_href}->{$page_name}
      = WiKicker::URI::make_read_page_url($self->script_url, $page_name);
  }
}

sub make_page_edit_href {
  my ($self, $page_name) = @_;

  if (exists $self->{page_edit_href}->{$page_name}) {
    return $self->{page_edit_href}->{$page_name};
  }
  else {
    return $self->{page_edit_href}->{$page_name} = $self->script_url .
      '?edit=' . WiKicker::URI::escape_URI($page_name);
  }
}

sub make_dangling_link {
  my ($self, $page_name, $content) = @_;

  my $href = $self->make_page_edit_href($page_name);
  my $content_string = WiKicker::HTML::escape($content || $page_name);
  return qq(<a href="$href">?</a><span class="DanglingLink">$content_string</span>);
}

sub make_fragment_identifier_link {
  my ($self, $fragment_identifier, $content) = @_;

  my $href = '#' . WiKicker::URI::escape_URI($fragment_identifier);
  my $content_string = WiKicker::HTML::escape($content || $fragment_identifier);
  return qq(<a href="$href">$content_string</a>);
}

sub _make_time_str {
  my ($self, $time) = @_;

  my $time_str;
  $time_str = WiKicker::Time::japanese_date_str($time, $self->timezone);
  return qq(<span class="date">$time_str</span>);
}

sub make_exists_page_link {
  my ($self, $page_name, $content, $mtime_flag, $subject_flag) = @_;

  my $base = $self->_make_exists_page_link_base($page_name, $content);
  unless ($mtime_flag || $subject_flag) {
    return $base;
  }
  else {
    my $time_str = '';
    if ($mtime_flag) {
      my $time = $self->db->last_modified($page_name);
      $time_str = $self->_make_time_str($time);
    }
    my $subject = '';
    if ($subject_flag) {
      $subject = $self->db->page_subject($page_name);
      if ($subject) {
        $subject = ' - ' . WiKicker::HTML::escape($subject);
      }
    }
    if ($time_str || $subject) {
      return qq($base <small>$time_str$subject</small>);
    }
    else {
      return $base;
    }
  }
}

sub make_pseudo_page_link {
  my ($self, $page_name, $content) = @_;

  my $href = $self->make_page_read_href($page_name);
  $content = WiKicker::HTML::escape($content || $page_name);
    
  return qq(<a href="$href">$content</a>);
}


sub _make_exists_page_link_base {
  my ($self, $page_name, $content) = @_;

  
  if ((! $content) && exists $self->{_page_link_base_cache}->{$page_name}) {
    return $self->{_page_link_base_cache}->{$page_name};
  }
  
  my $href = $self->make_page_read_href($page_name);
  $content = WiKicker::HTML::escape($content || $page_name);
  #my $subject = WiKicker::HTML::escape_attribute($self->db->page_subject($page_name));
  #my $link = qq(<a href="$href" title="$subject">$content</a>);
  my $link = qq(<a href="$href">$content</a>);
  if (!$content) {
    $self->{_page_link_base_cache}->{$page_name} = $link;
  }
  return $link;
}

sub make_page_name_link {
  my ($self, $page_name, $fragment_identifier, $content, $mtime_flag, $subject_flag) = @_;

  $page_name = WiKicker::PageName::normalize($page_name);
  
  # fragment identifier only?
  if ((! $page_name) && $fragment_identifier) {
    return $self->make_fragment_identifier_link($fragment_identifier, $content);
  }

  # exists page?
  if ($self->db->exists_live_page($page_name)) {
    return $self->make_exists_page_link($page_name, $content, $mtime_flag, $subject_flag);
  }

  # pseudo page?
  if ($self->configuration->is_pseudo_page($page_name)) {
    return $self->make_pseudo_page_link($page_name, $content);
  }

  #my @full_page_name_list = $self->db->full_page_name_list($page_name);
  my @full_page_name_list = $self->_get_full_page_name_list($page_name);

  # exists one match page.
  if (@full_page_name_list == 1) {
    return $self->
      make_exists_page_link($full_page_name_list[0],
                            $content || $page_name,
                            $mtime_flag, $subject_flag);
  }

  # exists some match pages.
  if (@full_page_name_list > 1) {
    my $content = WiKicker::HTML::escape($content || $page_name);
    my $result = $content;
    my $i = 0;
    for my $full_name (@full_page_name_list) {
      $i++;
      my $href = $self->make_page_read_href($full_name);
      my $title = WiKicker::HTML::escape($full_name);
      $result .= qq([<a href="$href" title="$title">$i</a>]);
    }
    $result .= ' ';
    return $result;
  }

  # dangling link
  return $self->make_dangling_link($page_name, $content);
}

sub _get_full_page_name_list {
  my ($self, $page_name) = @_;

  if (exists $self->{_full_page_name_list_cache}->{$page_name}) {
    return @{$self->{_full_page_name_list_cache}->{$page_name}};
  }
  my @full_page_name_list = $self->db->full_page_name_list($page_name);
  $self->{_full_page_name_list_cache}->{$page_name} = [ @full_page_name_list ];

  return @full_page_name_list;
}

1;

