#
# WiKicker
#
#  File.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: File.pm,v 1.2 2003/09/16 15:16:24 naney Exp $
#
#

package WiKicker::Tie::File;

use 5.005;
use strict;
use WiKicker::File;
use File::Path;
use Carp;

sub new {
  my ($class, $directory, $options) = @_;
  my $self = bless { directory => $directory,
                     options => $options
                   }, $class;

  $self->_initialize;

  return $self;
}

sub _initialize {
  my $self = shift;
  unless(-d $self->{directory}) {
    #mkdir($self->{directory}, 0777) || croak "Can't mkdir: $!";
    mkpath([ $self->{directory} ], 0, 0777);
  }
}

sub TIEHASH {
  my $class = shift;

  return $class->new(@_);
}

sub FETCH {
  my ($self, $key) = @_;

  $self->shared_lock_key($key);
  my $value;
  #warn "open: " . $self->file_full_name($key);
  if (open(FILE, $self->file_full_name($key))) {
    local $/;
    $value = <FILE>;
    close(FILE);
  }
  $self->unlock_key($key);
  
  return $value;
}

sub STORE {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 0);
}

sub append {
  my ($self, $key, $value) = @_;

  $self->_store($key, $value, 1);
}

sub _write_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">$file")) {
    binmode(FILE);
    print FILE $value;
    close(FILE);
    return 1;
  }
  else {
    return 0;
  }
}

sub _append_file {
  my ($self, $file, $value) = @_;
  if (open(FILE, ">>$file")) {
    print FILE $value;
    close(FILE);
    return 1;
  }
  else {
    return 0;
  }
}

sub _store {
  my ($self, $key, $value, $append) = @_;
  $self->exclusive_lock_key($key);
  my $file = $self->file_full_name($key);
  unless ($append) {
    if ($self->_write_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't store data to: $file";
    }
  }
  else {
    if ($self->_append_file($file, $value)) {
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      croak "Can't append data to: $file";
    }
  }
}

sub EXISTS {
  my ($self, $key) = @_;
  my $file = $self->file_full_name($key);
  if (-e $file) {
    return 1;
  }
  else {
    return 0;
  }
}

sub DELETE {
  my ($self, $key) = @_;

  $self->_delete($key);
}

sub _delete {
  my ($self, $key) = @_;

  $self->_delete_work_file($key);
}

sub _delete_work_file {
  my ($self, $key) = @_;

  my $file = $self->file_full_name($key);
  unlink($file);
}

sub CLEAR {
  my $self = shift;

  unless ($self->{CLOBER} > 1) {
    die;
  }
  my @all_keys = $self->all_keys;
  for my $key (@all_keys) {
    $self->_delete_work_file($key);
  }
}

sub FIRSTKEY {
  my $self = shift;

  @{$self->{key_list}} = $self->all_keys;
  return shift @{$self->{key_list}};
}

sub NEXTKEY {
  my $self = shift;

  return shift @{$self->{key_list}};
}

sub all_keys {
  my $self = shift;

  #warn "process $$: all keys called";

  opendir(DIR, $self->{directory}) || croak "Can't open directory: $!";
  #my @result =
  #  map { file_name_to_key($_) }
  #  grep { ! ($_ eq '.' || $_ eq '..' ||
  #            $_ eq 'RCS' || $_ =~ /-lock$/) } readdir(DIR);
  my @result;
  my $file;
  while (defined ($file = readdir(DIR))) {
    next if $file eq '.';
    next if $file eq '..';
    next if $file eq 'RCS';
    next if $file =~ /-lock$/;
    push(@result, file_name_to_key($file));
  }
  closedir(DIR);

  return @result;
}

sub all_keys_with_predicate {
  my ($self, $predicate) = @_;

  return grep { &$predicate($_) } $self->all_keys;
}

sub DESTROY {
  my $self = shift;
}

sub file_full_name {
  my ($self, $key) = @_;

  return $self->{directory} . '/' . key_to_file_name($key);
}

sub lock_file_name {
  my ($self, $key) = @_;
  return $self->file_full_name($key) . '-lock';
}

sub all_lock_file_name {
  my $self = shift;
  return $self->{directory} . '/' . 'alllock';
}

# class method
sub key_to_file_name {
#   my $key = shift;

#   my $name = encode_base64($key, '');
#   chomp $name;
#   $name =~ tr|/|.|;
#   return $name;
  return WiKicker::File::key_to_file_name(shift);
}

# class method
sub file_name_to_key {
  #my $file_name = shift;
  #$file_name =~ tr|.|/|;
  #return decode_base64($file_name);
  return WiKicker::File::file_name_to_key(shift);
}

# for override.
sub shared_lock_key {
}

# for override.
sub exclusive_lock_key {
}

# for override;
sub unlock_key {
}

1;
