#
# WiKicker
#
#  HtmlFragmentVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HtmlFragmentVisitor.pm,v 1.31 2004/01/29 14:40:57 naney Exp $
#
#

package WiKicker::WikiPage::HtmlFragmentVisitor;

use strict;
use WiKicker::URI;
use WiKicker::HTML;
use WiKicker::HTML::LinkMaker;
use WiKicker::WikiPage::Visitor;
use AutoLoader 'AUTOLOAD';
use Carp;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::Visitor);

sub new {
  my ($class, %param) = @_;

  my $self = bless { content => '',
                     comment_number => -1,
                   }, $class;

  for my $key (keys %param) {
    $self->{$key} = $param{$key};
  }

  return $self;
}

sub DESTROY {
  my $self = shift;

  #IFDEBUG: WiKicker::cycle_check($self);
  #IFDEBUG: WiKicker::total_size("HtmlFragmentVisitor(before DESTROY)", $self);
}

sub flush_closure {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{flush_closure} = $value;
  }
  return $self->{flush_closure};
}

sub in_inline {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->env->in_inline($value);
  }
  return $self->env->in_inline;
}

sub form_charset {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{form_charset} = $value;
  }
  return $self->{form_charset};
}

sub no_edit_section {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{no_edit_section} = $value;
  }
  return $self->{no_edit_section};
}

sub no_automatic_link {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{no_automatic_link} = $value;
  }
  return $self->{no_automatic_link};
}

sub word_extractor {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{word_extractor} = $value;
  }
  return $self->{word_extractor};
}

sub env {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{env} = $value;
  }
  return $self->{env};
}

sub wri_factory {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{wri_factory} = $value;
  }
  return $self->{wri_factory};
}

sub call_flush_closure {
  my $self = shift;

  if (defined $self->flush_closure) {
    &{$self->flush_closure}($self->content);
    $self->{content} = '';
  }
}

sub _initialize {
  my $self = shift;

  croak "no env" unless defined $self->env;

  unless ($self->no_automatic_link) {
    eval "use WiKicker::Text::WordExtractor";
    if ($@) {
      croak "Can't load WiKicker::Text::WordExtractor: $@";
    }
    $self->word_extractor(WiKicker::Text::WordExtractor->new);
    if ($self->env->db) {
      $self->word_extractor->set_words([$self->env->db->page_list]);
    }
  }

  $self->env->link_maker(WiKicker::HTML::LinkMaker->new(env => $self->env));

  $self->{anchor} = 0;
  $self->{section} = 0;
  $self->in_inline(0);
}

sub visit_DocumentNode {
  my ($self, $document_node) = @_;

  $self->_initialize;

  {
    my @nodes = $document_node->nodes;
    if (@nodes > 0) {
      my $node = $nodes[0];
      unless ($node->isa('WiKicker::WikiPage::HeadingNode')) {
        $self->_start_article;
        $self->_start_article_body;
        $self->{not_first_article} = 1;
      }
    }
  }

  for my $node ($document_node->nodes) {
    $node->accept($self);
    $self->call_flush_closure;
  }
  $self->_add_edit_section_div($self->{section});

  if ($self->{not_first_article}) {
    $self->_end_section;
    $self->_end_article_body;
    $self->_end_article;
  }
  $self->call_flush_closure;
}

sub _start_article {
  my $self = shift;

  # for hook
}

sub _end_article {
  my $self = shift;

  #for hook
}

sub _start_article_body {
  my $self = shift;

  # for hook
}

sub _end_article_body {
  my $self = shift;

  # for hook
}

sub _start_section {
  my $self = shift;

  # for hook
}

sub _end_section {
  my $self = shift;

  # for hook
}

sub visit_ParagraphNode {
  my ($self, $paragraph_node) = @_;

  my @nodes = $paragraph_node->nodes;
  if ($paragraph_node->is_index_or_form_only) {
    $self->in_inline(0);
    $nodes[0]->accept($self);
    return;
  }
  
  $self->{content} .= '<p>';
  $self->in_inline(1);
  for my $node (@nodes) {
    $node->accept($self);
  }
  $self->in_inline(0);
  $self->{content} .= "</p>\n";
}

sub visit_EmphasisNode {
  my ($self, $emphasis_node) = @_;
  $self->{content} .= '<em>';
  for my $node ($emphasis_node->nodes) {
    $node->accept($self);
  }
  $self->{content} .= '</em>';
}

sub visit_HeadingNode {
  my ($self, $heading_node) = @_;

  if ($heading_node->level <= 2) {
    $self->_add_edit_section_div($self->{section});
    $self->{section}++;
  }
  $self->{anchor}++;

  if ($heading_node->level == 1) {
    if ($self->{not_first_article}) {
      $self->_end_section;
      $self->_end_article_body;
      $self->_end_article;
    }
    $self->{not_first_article} = 1;
    $self->_start_article;
  }
  elsif ($heading_node->level == 2) {
    $self->_end_section;
    $self->_start_section;
  }

  my $level = $heading_node->level() + 1;

  my $anchor_name = $self->{anchor};
  $self->{content} .= qq(<h$level><a name="h$anchor_name"></a>);
  $self->in_inline(1);
  for my $node ($heading_node->nodes) {
    $node->accept($self);
  }
  chomp $self->{content};
  $self->in_inline(0);
  $self->{content} .= qq(</h$level>\n);

  if ($heading_node->level == 1) {
    $self->_start_article_body;
  }
}


sub visit_TextNode {
  my ($self, $text) = @_;

  if ($self->no_automatic_link) {
    $self->{content} .= WiKicker::HTML::escape($text->to_string);
  }
  else {
    $self->_visit_TextNode_for_automatic_link($text);
  }
}

sub _visit_TextNode_for_automatic_link {
  my ($self, $text) = @_;

  my $splited = $self->word_extractor->split_into_words($text->to_string);
  for my $item (@$splited) {
    if ($item->[0] == 0) {
      $self->{content} .= WiKicker::HTML::escape($item->[1]);
    }
    else {
      my $page_name = $item->[1];
      $page_name = $item->[2] if defined $item->[2];
      my $href = $self->env->link_maker->make_page_read_href($page_name);
      my $link_text = WiKicker::HTML::escape($item->[1]);
      $self->{content} .= qq(<a class="auto" href="$href">$link_text</a>);
    }
  }
}

sub visit_VerbatimNode {
  my ($self, $node) = @_;

  $self->{content} .= WiKicker::HTML::escape($node->text);
}

sub visit_WriNode {
  my ($self, $wri_node) = @_;

  $self->_add_wri_content($wri_node->wri);
}

sub visit_BracketNameNode {
  my ($self, $bracket_name_node) = @_;

  if ($bracket_name_node->is_empty_bracket_name) {
    $self->{content} .= $bracket_name_node->original;
  }
  else {
    $self->_add_wri_content($bracket_name_node->wri,
                            $bracket_name_node->content);
  }
}

sub visit_HrNode {
  my ($self, $hr_node) = @_;

  $self->{content} .= "<hr>\n";
}

sub visit_EmptyLinesNode {
  my ($self, $empty_lines_node) = @_;

  # nothing to do
}

sub visit_ListNode {
  my ($self, $list_node) = @_;

  my $element;
  if ($list_node->ordered) {
    $element = 'ol';
  }
  else {
    $element = 'ul';
  }

  $self->{content} .= '    ' x ($list_node->level - 1);
  $self->{content} .= qq(<$element>\n);

  for my $node ($list_node->nodes) {
    $node->accept($self);
  }

  $self->{content} .= '    ' x ($list_node->level - 1);
  $self->{content} .= "</$element>\n";
  $self->in_inline(0);
}

sub _visit_normal_node {
  my ($self, $node, $element_name) = @_;

  $self->{content} .= "<$element_name>";
  for my $child ($node->nodes) {
    $child->accept($self);
  }
  $self->{content} .= "</$element_name>";
}

sub _visit_normal_chomp_node {
  my ($self, $node, $element_name) = @_;

  $self->{content} .= "<$element_name>";
  for my $child ($node->nodes) {
    $child->accept($self);
  }
  chomp $self->{content};
  $self->{content} .= "</$element_name>";
}

sub visit_ListItemNode {
  my ($self, $node) = @_;

  $self->{content} .= '  ';
  $self->{content} .= '    ' x ($node->level - 1);
  $self->in_inline(0);
  $self->_visit_normal_chomp_node($node, 'li');
  $self->{content} .= "\n";

  $self->call_flush_closure; # for long list
}  

sub visit_TableNode {
  my ($self, $node) = @_;

  $self->_visit_normal_node($node, 'table');
  $self->{content} .= "\n";
}

sub visit_TableRecordNode {
  my ($self, $record_node) = @_;

  
  $self->{content} .= qq(<tr>);

  for my $node ($record_node->nodes) {
    $node->accept($self);
  }
  $self->{content} .= qq(</tr>\n);
}

sub visit_TableCellNode {
  my ($self, $cell_node) = @_;

  my $align = $cell_node->align;
  my $align_attribute = '';
  if ($align eq 'center') {
    $align_attribute = qq( align="center");
  }
  elsif ($align eq 'right') {
    $align_attribute = qq( align="right");
  }
  
  $self->{content} .= qq(<td$align_attribute>);
  $self->in_inline(0);
  for my $node ($cell_node->nodes) {
    $node->accept($self);
  }
  $self->{content} .= qq(</td>);
}

sub _add_wri_content {
  my ($self, $wri, $content) = @_;

  $self->{content} .= $wri->to_html($content, $self->env);
}

sub content {
  return shift->{content};
}

# @todo same WiKicker::CGI::View::HTML::Base::edit_url
sub _edit_url {
  my $self = shift;

  $self->env->wikicker_url . '?edit=' . WiKicker::URI::escape_URI($self->env->page_name);
}

sub _add_edit_section_div {
  my ($self, $section_number) = @_;

  unless ($self->no_edit_section) {
    my $section_url = $self->_edit_url . '&amp;section=' . $section_number;

    $self->{content}
      .= qq(<div class="section-toolbar">[<a href="$section_url" title="edit this section">edit</a>]</div>\n);
  }
}

sub visit_PreNode;
sub visit_QuoteNode;
sub visit_DlNode;
sub visit_DtNode;
sub visit_DdNode;
sub visit_QuoteLineNode;

1;


__END__
sub visit_PreNode {
  my ($self, $pre_node) = @_;

  $self->{content} .= '<pre>';
  $self->in_inline(1);
  $self->{content} .= WiKicker::HTML::escape($pre_node->to_string);
  $self->in_inline(0);
  $self->{content} .= "</pre>\n";
}

sub visit_QuoteNode {
  my ($self, $node) = @_;

  my $in_p = 0;
  
  $self->{content} .= '<blockquote>';
  for my $child ($node->nodes) {
    if ($child->isa('WiKicker::WikiPage::QuoteNode')) {
      if ($in_p) {
        $self->{content} .= '</p>';
        $in_p = 0;
        $self->in_inline(0);
      }
    }
    else {
      unless ($in_p) {
        $self->{content} .= '<p>';
        $in_p = 1;
        $self->in_inline(1);
      }
    }
    $child->accept($self);
  }
  if ($in_p) {
    $self->in_inline(0);
    $self->{content} .= '</p>';
  }
  $self->{content} .= "</blockquote>\n";
}

sub visit_QuoteLineNode {
  my ($self, $line_node) = @_;
  for my $node ($line_node->nodes) {
    $node->accept($self);
  }
}

sub visit_DlNode {
  my ($self, $dl_node) = @_;

  $self->{content} .= '    ' x ($dl_node->level - 1);
  $self->{content} .= qq(<dl>\n);
  for my $node ($dl_node->nodes) {
    $node->accept($self);
  }
  $self->{content} .= '    ' x ($dl_node->level - 1);
  $self->{content} .= "</dl>\n";
}

sub visit_DtNode {
  my ($self, $node) = @_;

  my $child_string = $node->get_child_string;
  unless ($child_string =~ /^\s*$/) {
    $self->{content} .= '  ';
    $self->{content} .= '    ' x ($node->level - 1);
    $self->in_inline(1);
    $self->_visit_normal_chomp_node($node, 'dt');
    $self->in_inline(0);
    $self->{content} .= "\n";
  }
}

sub visit_DdNode {
  my ($self, $node) = @_;

  $self->{content} .= '  ';
  $self->{content} .= '    ' x ($node->level - 1);
  $self->in_inline(0);
  $self->_visit_normal_chomp_node($node, 'dd');
  $self->{content} .= "\n";
}

